(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.SavVui = global.SavVui || {})));
}(this, (function (exports) { 'use strict';

function trust (val) {
  return val && val !== 'false' && val !== '0'
}

var common = {
  props: {
    color: {
      type: [Boolean, String],
      default: false
    },
    size: {
      type: [Boolean, String],
      default: false
    },
    disabled: {
      type: [Boolean, String],
      default: false
    },
    flex: {
      type: [Boolean, String],
      default: false
    },
    col: {
      type: [Number, String],
      default: 0
    },
    offset: {
      type: [Number, String],
      default: 0
    },
    value: {
      type: [String, Number, Boolean, Object, Array],
      default: ''
    },
    textField: {
      type: String,
      default: 'text'
    },
    valueField: {
      type: String,
      default: 'value'
    },
    options: {
      type: Array,
      default: Array
    },
    option: {
      type: Object,
      default: Object
    },
    type: {
      type: [Number, String],
      default: 0
    },
    align: {
      type: String,
      default: ''
    },
    block: {
      type: Boolean,
      default: false
    },
    vertical: {
      type: Boolean,
      default: false
    }
  },
  computed: {
    sizeModify () {
      return trust(this.size) ? `is-${this.size}` : ''
    },
    colModify () {
      return trust(this.col) ? `is-${this.col}` : ''
    },
    offsetModify () {
      return trust(this.offset) ? `is-offset-${this.offset}` : ''
    },
    colorModify () {
      return trust(this.color) ? `is-${this.color}` : ''
    },
    disabledModify () {
      return trust(this.disabled) ? `is-disabled` : ''
    },
    flexModify () {
      return trust(this.flex) ? `is-${this.flex}` : ''
    },
    typeModify () {
      return trust(this.type) ? `is-${this.type}` : ''
    },
    alignModify () {
      return trust(this.align) ? `is-${this.align}` : ''
    },
    blockModify () {
      return trust(this.block) ? `is-block` : ''
    },
    verticalModify () {
      return trust(this.vertical) ? `is-vertical` : ''
    }
  }
};

function createMixins (types) {
  let ret = {
    props: {},
    computed: {}
  };
  let hasProps = false;
  let hasComputed = false;
  for (let it of types) {
    if (common.props[it]) {
      ret.props[it] = common.props[it];
      hasProps = true;
    }
    let modify = `${it}Modify`;
    if (common.computed[modify]) {
      ret.computed[modify] = common.computed[modify];
      hasComputed = true;
    }
  }
  if (!hasProps) {
    delete ret.props;
  }
  if (!hasComputed) {
    delete ret.computed;
  }
  return [ret]
}

const option = createMixins(['textField', 'valueField', 'value']);
const options = createMixins(['textField', 'valueField', 'options', 'value']);
const elements = createMixins(['color', 'size', 'disabled']);

var SavBtn = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return (_vm.to)?_c('router-link',{class:['sav-btn', _vm.disabledModify, _vm.colorModify, _vm.sizeModify],attrs:{"to":_vm.to}},[_vm._t("default")],2):_c('button',{class:['sav-btn', _vm.disabledModify, _vm.colorModify, _vm.sizeModify, {'is-loading':_vm.loading}],attrs:{"disabled":_vm.disabled}},[_vm._t("default")],2)},
staticRenderFns: [],
    mixins: elements,
    props: {
      to: {
        type: [Boolean, Object],
        default: false
      },
      loading: {
        type: [Boolean],
        default: false
      }
    }
  };

var SavBtnGroup = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('span',{class:['sav-btn-group', _vm.dirModify]},[_vm._t("default")],2)},
staticRenderFns: [],
    mixins: createMixins(['vertical']),
    computed: {
      dirModify () {
        return this.verticalModify || 'is-horizontal'
      }
    }
  };

// 并不需要直接包装input
  var SavInput = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('input',{staticClass:"sav-input"})},
staticRenderFns: [],
  };

var SavText = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c("div")},
staticRenderFns: [],
  };

var SavCheck = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('label',{class:['sav-check', _vm.disabledModify, _vm.colorModify, _vm.sizeModify]},[_c('input',{attrs:{"type":"checkbox","disabled":_vm.disabled},domProps:{"checked":_vm.isChecked,"value":_vm.option},on:{"change":_vm.changeValue}}),_vm._v(" "),_c('span',{staticClass:"as-text"},[_vm._t("default")],2)])},
staticRenderFns: [],
    mixins: elements,
    props: {
      value: {
        type: [Array, Boolean],
        default: false
      },
      option: {
        default: null
      }
    },
    computed: {
      isOption () {
        return this.option !== null
      },
      isChecked () {
        if (this.isOption) {
          return this.value.indexOf(this.option) >= 0
        }
        return !!this.value
      }
    },
    methods: {
      changeValue (val) {
        if (this.isOption) {
          let arr;
          if (this.isChecked) {
            arr = this.value.filter((it) => it != this.option);
          } else {
            arr = this.value.concat(this.option);
          }
          this.$emit('input', arr);
        } else {
          this.$emit('input', !this.value);
        }
      }
    }
  };

let incId = 0;
  var SavCheckGroup = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('span',{staticClass:"sav-check-group"},_vm._l((_vm.options),function(opt){return _c('sav-check',{directives:[{name:"bridge",rawName:"v-bridge"}],attrs:{"disabled":_vm.disabled,"color":_vm.color,"size":_vm.size,"value":_vm.value,"option":_vm.isTiny ? opt : opt[_vm.valueField]}},[_vm._v(_vm._s(_vm.isTiny ? opt : opt[_vm.textField]))])}))},
staticRenderFns: [],
    mixins: [...elements, ...options],
    props: {
      inputName: {
        type: String,
        default: ''
      },
      tiny: {
        type: [Boolean, String, Number],
        default: false
      }
    },
    computed: {
      isTiny () {
        return trust(this.tiny)
      }
    },
    data () {
      return {
        innerName: 'scn-' + incId++
      }
    }
  };

var SavRadio = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('label',{class:['sav-radio', _vm.disabledModify, _vm.colorModify, _vm.sizeModify, _vm.blockModify]},[_c('input',{attrs:{"type":"radio","disabled":_vm.disabled},domProps:{"checked":_vm.isOption ? _vm.value == _vm.option : _vm.value,"value":_vm.option},on:{"change":_vm.changeValue}}),_vm._v(" "),_c('span',{staticClass:"as-text"},[_vm._t("default")],2)])},
staticRenderFns: [],
    mixins: elements.concat(createMixins(['block'])),
    props: {
      value: {
        default: false
      },
      option: {
        default: null
      }
    },
    computed: {
      isOption () {
        return this.option !== null
      }
    },
    methods: {
      changeValue (val) {
        this.$emit('input', this.isOption ? this.option : !this.value);
      }
    }
  };

let incId$1 = 0;
  var SavRadioGroup = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('span',{class:['sav-radio-group', _vm.verticalModify]},_vm._l((_vm.options),function(opt){return _c('label',{class:['sav-radio', _vm.disabledModify, _vm.colorModify, _vm.sizeModify]},[_c('input',{attrs:{"type":"radio","name":_vm.inputName || _vm.innerName,"disabled":_vm.disabled},domProps:{"value":opt[_vm.valueField],"checked":_vm.value == opt[_vm.valueField]},on:{"change":function($event){_vm.changeValue(opt[_vm.valueField]);}}}),_vm._v(" "),_c('span',{staticClass:"as-text"},[_vm._v(_vm._s(opt[_vm.textField]))])])}))},
staticRenderFns: [],
    mixins: elements.concat(options, createMixins(['vertical'])),
    props: {
      inputName: {
        type: String,
        default: ''
      }
    },
    data () {
      return {
        innerName: 'srn-' + incId$1++
      }
    },
    methods: {
      changeValue (value) {
        this.$emit('input', value);
      }
    }
  };

var SavIcon = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('i',{class:['as-icon', _vm.icon, _vm.colorModify, _vm.sizeModify]})},
staticRenderFns: [],
    mixins: createMixins(['color', 'size']),
    props: {
      icon: {
        type: String,
        default: ''
      }
    }
  };

var SavField = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return (_vm.isLabel)?_c('label',{staticClass:"sav-field"},[_vm._t("title",[_c('span',{staticClass:"sav-label"},[_vm._v(_vm._s(_vm.labelText))])]),_vm._v(" "),_c('div',{staticClass:"field-body"},[_vm._t("default")],2)],2):_c('div',{staticClass:"sav-field"},[_vm._t("title",[_c('span',{staticClass:"sav-label"},[_vm._v(_vm._s(_vm.labelText))])]),_vm._v(" "),_c('div',{staticClass:"field-body"},[_vm._t("default")],2)],2)},
staticRenderFns: [],
    props: {
      isLabel: {
        type: Boolean,
        default: true
      },
      labelText: {
        type: String,
        default: ''
      }
    }
  };

var SavForm = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{class:['sav-flex-block', _vm.alignModify]},[_c('form',{class:['sav-form', _vm.alignModify, _vm.horizontalModify, _vm.growModify]},[_vm._t("default")],2)])},
staticRenderFns: [],
    mixins: createMixins(['align']),
    props: {
      align: {
        default: 'center'
      },
      horizontal: {
        type: [Boolean, Number, String],
        default: false
      },
      grow: {
        type: [Number, String],
        default: 0
      }
    },
    computed: {
      horizontalModify () {
        return trust(this.horizontal) ? `is-horizontal` : 'is-vertical'
      },
      growModify () {
        return trust(this.grow) ? `has-grow-${this.grow}` : ''
      }
    }
  };

var SavTile = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{class:['sav-tile', _vm.colModify, _vm.flexModify, {
    'is-block': !_vm.flex && _vm.block,
    'is-vertical': _vm.vertical
  }]},[_vm._t("default")],2)},
staticRenderFns: [],
    mixins: createMixins(['flex', 'col']),
    props: {
      block: {
        default: false
      },
      vertical: {
        default: false
      }
    }
  };

var SavDropdown = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{class:['sav-dropdown', _vm.openModify, _vm.hoverModify, {'is-fill': _vm.fill}]},[_vm._t("default"),_vm._v(" "),_c('div',{staticClass:"as-dropview"},[_vm._t("dropview")],2)],2)},
staticRenderFns: [],
    props: {
      open: {
        type: [Boolean, String],
        default: false
      },
      hover: {
        type: [Boolean, String],
        default: false
      },
      fill: {
        type: [Boolean, String],
        default: false
      }
    },
    computed: {
      openModify () {
        return trust(this.open) ? `is-open` : ''
      },
      hoverModify () {
        return trust(this.hover) ? `has-hover-open` : ''
      }
    }
  };

var SavSelect = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('sav-dropdown',{attrs:{"open":_vm.innerOpen,"fill":_vm.fill},nativeOn:{"mouseenter":function($event){_vm.disabled || _vm.hover && (_vm.innerOpen = true);},"mouseleave":function($event){_vm.disabled || (_vm.innerOpen = false);}}},[_c('sav-btn',{class:[_vm.colorModify, _vm.sizeModify],attrs:{"disabled":_vm.disabled},nativeOn:{"click":function($event){_vm.innerOpen = !_vm.innerOpen;}}},[_c('span',[_vm._v(_vm._s(_vm.textSelected || _vm.placeholder))]),_vm._v(" "),_c('sav-icon',{attrs:{"icon":"is-icon-caret"}})],1),_vm._v(" "),_c('ul',{staticClass:"is-drop-select",slot:"dropview"},_vm._l((_vm.options),function(opt){return _c('li',{key:"valueField",class:['as-item', {
        'is-active': opt[_vm.valueField] == _vm.value
      }],on:{"click":function($event){_vm.selectItem(opt);}}},[_c('span',[_vm._v(_vm._s(opt[_vm.textField]))]),_vm._v(" "),(opt[_vm.valueField] == _vm.value)?_c('sav-icon',{class:[_vm.colorModify],attrs:{"icon":"is-icon-right"}}):_vm._e()],1)}))],1)},
staticRenderFns: [],
    mixins: createMixins(['color', 'size', 'textField', 'valueField', 'options', 'value']),
    components: {
      SavBtn,
      SavIcon,
      SavDropdown
    },
    data () {
      return {
        innerOpen : false
      }
    },
    computed: {
      textSelected () {
        let text;
        this.options.some((it) => {
          if (it[this.valueField] == this.value) {
            text = it[this.textField];
            return true
          }
        });
        return text
      }
    },
    props: {
      placeholder: {
        type: String,
        default: '请选择'
      },
      hover: {
        type: [Boolean, String],
        default: false
      },
      disabled: {
        type: Boolean,
        default: false
      },
      fill: {
        type: [Boolean, String],
        default: false
      }
    },
    methods: {
      selectItem (item) {
        this.innerOpen = false;
        this.$emit('input', item.value);
      }
    }
  };

var SavRow = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{class:['sav-row', _vm.isFloatModify, _vm.colModify]},[_vm._t("default")],2)},
staticRenderFns: [],
    props: {
      col: {
        type: [Number, String],
        default: 0
      },
      float : {
        default: false
      }
    },
    computed: {
      colModify () {
        return trust(this.col) ? `has-${this.col}` : ''
      },
      isFloatModify () {
        return trust(this.float) ? `is-float` : ''
      }
    }
  };

var SavCol = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{class:['sav-col', _vm.colModify, _vm.offsetModify]},[_vm._t("default")],2)},
staticRenderFns: [],
    mixins: createMixins(['col', 'offset'])
  };

var SavTab = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{class:['sav-tab']},[_c('div',{class:['sav-tab-nav', _vm.colorModify, _vm.sizeModify, _vm.flexModify, _vm.typeModify]},_vm._l((_vm.options),function(option$$1){return _c(_vm.navComponent,{tag:"component",class:['as-item', {
        'is-active': option$$1[_vm.valueField] == _vm.value
      }],attrs:{"value":_vm.value,"option":option$$1,"textField":_vm.textField,"valueField":_vm.valueField},on:{"change":_vm.changeTab}})})),_vm._v(" "),_c('div',{staticClass:"sav-tab-contents"},_vm._l((_vm.options),function(option$$1){return _c(_vm.contentComponent,{directives:[{name:"show",rawName:"v-show",value:(option$$1[_vm.valueField] == _vm.value),expression:"option[valueField] == value"}],key:option$$1[_vm.valueField],tag:"component",class:['as-item', {
        'is-active': option$$1[_vm.valueField] == _vm.value
      }],attrs:{"option":option$$1,"contentField":_vm.contentField}})}))])},
staticRenderFns: [],
componentMixins: [{
  name: 'navItem',
  render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('a',{on:{"click":function($event){$event.preventDefault();_vm.changeValue(_vm.option[_vm.valueField]);}}},[_vm._v(_vm._s(_vm.option[_vm.textField]))])},
  staticRenderFns: [],
},{
  name: 'contentItem',
  render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',[_vm._v(_vm._s(_vm.option[_vm.contentField]))])},
  staticRenderFns: [],
}],
    mixins: createMixins(['color', 'size', 'flex', 'type', 'value', 'options', 'textField', 'valueField']),
    props: {
      navComponent: {
        type: [String, Object],
        default: 'navItem'
      },
      contentComponent: {
        type: [String, Object],
        default: 'contentItem'
      },
      contentField: {
        type: String,
        default: 'content'
      }
    },
    methods: {
      changeTab (value) {
        this.$emit('input', value);
      }
    },
    components: {
      navItem: {
        mixins: createMixins(['value', 'option', 'textField', 'valueField']),
        methods: {
          changeValue (newValue) {
            if (this.value !== newValue) {
              this.$emit('change', newValue);
            }
          }
        }
      },
      contentItem: {
        mixins: createMixins(['option']),
        props: ['contentField']
      }
    }
  };

var SavTabNav = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{class:['sav-tab-nav', _vm.colorModify, _vm.sizeModify, _vm.flexModify, _vm.typeModify]},[_vm._t("default")],2)},
staticRenderFns: [],
    mixins: createMixins(['color', 'size', 'flex', 'type']),
  };

var SavTable = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('table',{staticClass:"sav-table"},[_c('thead',[_c('tr',_vm._l((_vm.columns),function(option$$1){return _c(_vm.columnComponent,{tag:"component",attrs:{"option":option$$1,"textField":_vm.textField,"valueField":_vm.valueField}})}))]),_vm._v(" "),_c('tbody',[_vm._t("default",_vm._l((_vm.rows),function(entry){return _c('tr',_vm._l((_vm.columns),function(option$$1){return _c(option$$1[_vm.componentField] || _vm.contentComponent,{tag:"component",attrs:{"option":option$$1,"entry":entry,"valueField":_vm.valueField}})}))}))],2)])},
staticRenderFns: [],
componentMixins: [{
  name: 'columnItem',
  render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('th',[_vm._v(_vm._s(_vm.option[_vm.textField]))])},
  staticRenderFns: [],
},{
  name: 'contentItem',
  render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('td',[_vm._v(_vm._s(_vm.entry[_vm.option[_vm.valueField]]))])},
  staticRenderFns: [],
}],
    mixins: createMixins(['size', 'textField', 'valueField']),
    props: {
      columns: {
        type: Array,
        default: Array
      },
      rows: {
        type: Array,
        default: Array
      },
      columnComponent: {
        type: [String, Object],
        default: 'columnItem'
      },
      contentComponent: {
        type: [String, Object],
        default: 'contentItem'
      },
      componentField: {
        type: String,
        default: 'component'
      }
    },
    components: {
      columnItem: {
        mixins: createMixins(['option', 'textField', 'valueField'])
      },
      contentItem: {
        mixins: createMixins(['option', 'valueField']),
        props: ['entry']
      }
    }
  };

var SavTree = {
render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',[_c('ul',{class:['sav-tree', _vm.isSize]},[_vm._m(0),_vm._v(" "),_vm._m(1),_vm._v(" "),_vm._m(2)])])},
staticRenderFns: [function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('li',{staticClass:"tree-node"},[_c('div',{staticClass:"tree-hold",attrs:{"unselectable":"on"}},[_vm._v(" ")]),_c('i',{staticClass:"tree-toggle"}),_vm._v(" "),_c('a',{staticClass:"as-item"},[_vm._v("单个子节点")])])},function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('li',{staticClass:"tree-node is-open is-active has-sub"},[_c('div',{staticClass:"tree-hold",attrs:{"unselectable":"on"}},[_vm._v(" ")]),_c('i',{staticClass:"tree-toggle"}),_vm._v(" "),_c('a',{staticClass:"as-item"},[_vm._v("展开的子节点")]),_vm._v(" "),_c('ul',{staticClass:"tree-child"},[_c('li',{staticClass:"tree-node"},[_c('div',{staticClass:"tree-hold",attrs:{"unselectable":"on"}},[_vm._v(" ")]),_c('i',{staticClass:"tree-toggle"}),_vm._v(" "),_c('a',{staticClass:"as-item"},[_vm._v("单个子节点")])]),_vm._v(" "),_c('li',{staticClass:"tree-node"},[_c('div',{staticClass:"tree-hold",attrs:{"unselectable":"on"}},[_vm._v(" ")]),_c('i',{staticClass:"tree-toggle"}),_vm._v(" "),_c('a',{staticClass:"as-item"},[_vm._v("展开的子节点")])])])])},function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('li',{staticClass:"tree-node is-closed has-sub"},[_c('div',{staticClass:"tree-hold",attrs:{"unselectable":"on"}},[_vm._v(" ")]),_c('i',{staticClass:"tree-toggle"}),_vm._v(" "),_c('a',{staticClass:"as-item"},[_vm._v("闭合的子节点")]),_vm._v(" "),_c('ul',{staticClass:"tree-child"},[_c('li',{staticClass:"tree-node"},[_c('div',{staticClass:"tree-hold",attrs:{"unselectable":"on"}},[_vm._v(" ")]),_c('i',{staticClass:"tree-toggle"}),_vm._v(" "),_c('a',{staticClass:"as-item"},[_vm._v("单个子节点")])]),_vm._v(" "),_c('li',{staticClass:"tree-node"},[_c('div',{staticClass:"tree-hold",attrs:{"unselectable":"on"}},[_vm._v(" ")]),_c('i',{staticClass:"tree-toggle"}),_vm._v(" "),_c('a',{staticClass:"as-item"},[_vm._v("展开的子节点")])]),_vm._v(" "),_c('li',{staticClass:"tree-node is-closed has-sub"},[_c('div',{staticClass:"tree-hold",attrs:{"unselectable":"on"}},[_vm._v(" ")]),_c('i',{staticClass:"tree-toggle"}),_vm._v(" "),_c('a',{staticClass:"as-item"},[_vm._v("闭合的子节点")]),_vm._v(" "),_c('ul',{staticClass:"tree-child"},[_c('li',{staticClass:"tree-node"},[_c('div',{staticClass:"tree-hold",attrs:{"unselectable":"on"}},[_vm._v(" ")]),_c('i',{staticClass:"tree-toggle"}),_vm._v(" "),_c('a',{staticClass:"as-item"},[_vm._v("单个子节点")])]),_vm._v(" "),_c('li',{staticClass:"tree-node"},[_c('div',{staticClass:"tree-hold",attrs:{"unselectable":"on"}},[_vm._v(" ")]),_c('i',{staticClass:"tree-toggle"}),_vm._v(" "),_c('a',{staticClass:"as-item"},[_vm._v("展开的子节点")])])])])])])}],
    props: ['model'],
    data () {
      return {
        open: false,
        isFolder: true
      }
    },
    computed: {
      isFolder () {
        return this.model.children && this.model.children.length
      }
    },
    created () {
      console.log(this.model);
    },
    methods: {
      toggle () {
        if (this.isFolder) {
          this.open = !this.open;
        }
      }
    }
  };

var focus = {
  inserted (el) {
    el.focus();
  }
};

/**
 * 组件事件桥接
 *   可以省略掉 v-on 的 $emit
 * v-bridge 默认桥接 input事件
 * v-bridge.change='input' 将change事件转换为input并emit, 也就是 $emit('input', ..args)
 */
var bridge = {
  bind (el, binding, {componentInstance, context}, newVNode) {
    if (!(componentInstance && context)) {
      // console.log(newVNode)
      return
    }
    let modifiers = Object.keys(binding.modifiers);
    modifiers.length || (modifiers.push('input'));
    modifiers.forEach((modify) => {
      let ret = componentInstance.$on(modify, function (...args) {
        args.unshift(binding.expression || modify);
        context.$emit.apply(context, args);
      });
    });
  }
};

/**
 * tooltip提示
 * v-tip="text"
 * v-tip="bindValue"
 * html
 * v-tip.html="bindValue"
 * align
 * v-tip.left="left-text"
 * v-tip.right="right-text"
 */

var tip = {
  bind: applyBinding,
  update: applyBinding
};

function applyBinding (el, binding) {
  let ds = parseBinding(binding);
  if (!(ds || el.dataset)) {
    return
  }
  Object.assign(el.dataset, ds);
}

const aligns = ['top', 'left', 'right', 'bottom'];

function parseBinding (binding) {
  let ret = {};
  let tipAlign = aligns.find((it) => binding.modifiers[it]);
  let isHtml = binding.modifiers.html;
  let tipValue = (binding.value !== undefined) ? binding.value : binding.expression;
  if (tipValue) {
    if (tipAlign) {
      ret.tipAlign = tipAlign;
    }
    if (isHtml) {
      ret.tipHtml = tipValue;
    } else {
      ret.tip = tipValue;
    }
    return ret
  }
}

var directives = {
  focus,
  bridge,
  tip
};

function componentMixin (Vue) {
  let extend = Vue.extend;
  Vue.extend = function (options) {
    if (typeof options === 'object') {
      if (options.componentMixins) {
        let components = options.components || (options.components = {});
        options.componentMixins.forEach((it) => {
          let name = getComponentName(components, it.name);
          it = Object.assign(components[name], it);
          // reference from => mixin
          if (it.extends) { // Component.childComponent
            let mixins = getComponentMixins(this, it.extends);
            it.mixins = [].concat(it.mixins || [], mixins);
          }
          if (it.export) {
            delete components[name];
            Vue.component(name, it);
          }
        });
        delete options.componentMixins;
      }
    }
    return extend.call(this, options)
  };
}

function getComponentMixins (Vue, refs) {
  let mixins = [];
  if (!Array.isArray(refs)) {
    refs = [refs];
  }
  for (let ref of refs) {
    let [name, childName] = ref.split('.');
    let component = Vue.component(name);
    if (!component) {
      throw new Error(`componentMixin unable to resolve component: ${name}`)
    }
    if (childName) {
      let opts;
      try {
        opts = component.extendOptions.components[childName];
        if (!opts) {
          throw 1
        }
      } catch (err) {
        throw new Error(`componentMixin unable to resolve child component: ${childName} of ${name}`)
      }
      mixins.push(opts);
    } else {
      let opts = Object.assign({}, component.extendOptions);
      delete opts['_Ctor'];
      mixins.push(opts);
    }
  }
  return mixins
}

const nameIter = [camelCase, pascalCase, hyphenCase];

function getComponentName (components, name) {
  if (name in components)
    return name
  for (let iter of nameIter) {
    name = iter(name);
    if (name in components)
      return name
  }
  components[name] = {};
  return name
}

/**
 * Camelize a hyphen-delmited string.
 */
const camelCaseRE = /[-_](\w)/g;
function camelCase (str) {
  return lcfirst(str.replace(camelCaseRE, (_, c) => c ? c.toUpperCase() : ''))
}

/**
 * Capitalize a string.
 */
function ucfirst (str) {
  return str.charAt(0).toUpperCase() + str.slice(1)
}

/**
 * UnCapitalize a string.
 */
function lcfirst (str) {
  return str.charAt(0).toLowerCase() + str.slice(1)
}

const replaceAZRE = /([A-Z])/g;

/**
 * Hyphenate a camelCase string.
 */
function hyphenCase (str) {
  return camelCase(str).replace(replaceAZRE, '-$1').toLowerCase()
}

function pascalCase (str) {
  return ucfirst(camelCase(str))
}

function initSavTip () {
  if (installed) {
    return
  }
  document.addEventListener('mousedown', click);
  document.addEventListener('mousemove', move);
}

let tip$1;
let tipTarget;
let tipVisible = false;
let installed = false;

function getTip () {
  if (!tip$1) {
    tip$1 = document.createElement('div');
    tip$1.className = 'sav-tooltip';
    tip$1.id = 'tip';
    document.body.appendChild(tip$1);
  }
  return tip$1
}

function click (event) {
  if (!(event.buttons === 1 || event.which === 1)) {
    return
  }
  if (tipVisible) {
    tipVisible = false;
    let {style} = getTip();
    style.transition = 'opacity 150ms';
    style.opacity = 0;
    setTimeout(() => {
      if (!tipTarget) {
        style.display = `none`;
      }
    }, 100);
  }
}

function move (event) {
  if (tipTarget) {
    if (event.target === tipTarget) {
      return
    }
    if (tipTarget.contains(event.target)) {
      if (!isTip(event.target.dataset)) {
        return
      }
    }
    tipTarget = null;
  }
  let tipNode = getTipNode(event.target);
  if (tipNode) {
    tipTarget = tipNode;
    let tip = getTip();
    let {x, y} = applyTip(tipTarget.dataset, tipTarget, tip);
    let {style} = tip;
    style.transition = 'opacity 150ms, transform 150ms';
    style.transform = `translate(${x}, ${y})`;
    style.display = `block`;
    style.opacity = 1;
    tipVisible = true;
  } else if (tipVisible) {
    let {style} = getTip();
    style.transition = 'opacity 150ms, transform 150ms';
    style.opacity = 0;
    setTimeout(() => {
      if (!tipTarget) {
        style.display = `none`;
      }
    }, 100);
    tipVisible = false;
  }
}

function getTipNode (node) {
  if (node.dataset) {
    return node.closest('[data-tip]') || node.closest('[data-tip-html]')
  }
}

function isTip ({tip, tipHtml}) {
  return !!(tip || tipHtml)
}

const clss = ['is-left', 'is-top', 'is-right', 'is-bottom'];

function applyTip (opts, target, tipNode) {
  if (opts.tip) {
    tipNode.innerText = opts.tip;
  } else if (opts.tipHtml) {
    tipNode.innerHTML = opts.tipHtml;
  }
  let x;
  let y;
  let {left, top, right, bottom} = tipTarget.getBoundingClientRect();
  let offset = opts.tipOffset || 4;
  let hx = Math.round((left + right) / 2);
  let hy = Math.round((bottom + top) / 2);
  x = `calc(-50% + ${hx}px)`;
  y = `calc(-50% + ${hy}px)`;
  switch (opts.tipAlign) {
    case 'left':
      x = `calc(-100% + ${Math.round(left - offset)}px)`;
      break
    case 'top':
      y = `calc(-100% + ${Math.round(top - offset)}px)`;
      break
    case 'bottom':
      y = `${offset + bottom}px`;
      break
    default:
      x = `${Math.round(right + offset)}px`;
      break
  }
  clss.forEach((cls) => {
    tipNode.classList.remove(cls);
  });
  tipNode.classList.add(`is-${opts.tipAlign || 'right'}`);
  return {x, y}
}

function install (Vue) {
  initSavTip();

  componentMixin(Vue);

  Object.keys(directives).forEach((it) => {
    Vue.directive(it, directives[it]);
  });
  Object.keys(components).forEach((it) => {
    Vue.component(it, components[it]);
  });
}

let components = {
  SavBtn,
  SavBtnGroup,
  SavInput,
  SavText,
  SavCheck,
  SavCheckGroup,
  SavRadio,
  SavRadioGroup,
  SavIcon,
  SavField,
  SavForm,
  SavSelect,
  SavTile,
  SavRow,
  SavCol,
  SavTab,
  SavTabNav,
  SavTable,
  SavTree,
  SavDropdown
};

Object.defineProperty(components, 'install', {
  value: install,
  enumerable: false,
  configurable: true
});

if (typeof window !== 'undefined' && window.Vue) {
  window.Vue.use(components);
}

exports.install = install;
exports['default'] = components;

Object.defineProperty(exports, '__esModule', { value: true });

})));
